@@ -3,7 +3,7 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -13,12 +13,4 @@
 #include "smyrnadefs.h"
 #include <glcomp/glcompset.h>
 
-#ifdef __cplusplus
-extern "C" {
-#endif
-
-    extern void menu_click_add_camera(void *p);
-
-#ifdef __cplusplus
-}				/* end extern "C" */
-#endif
+    extern void menu_click_add_camera(void);
