@@ -30,6 +30,7 @@
 #include <stddef.h>
 #include <stdio.h>
 #include <string.h>
+#include <time.h>
 #include <unistd.h>
 #include <util/agxbuf.h>
 #include <util/alloc.h>
@@ -75,6 +76,13 @@ typedef struct {
   int verbose;
 } options;
 
+static clock_t start_timer(void) { return clock(); }
+
+static double elapsed_sec(clock_t start) {
+  const clock_t end = clock();
+  return (end - start) / (double)CLOCKS_PER_SEC;
+}
+
 static FILE *openOut(char *name) {
   FILE *const outs = gv_fopen(name, "w");
   if (outs == 0) {
@@ -83,8 +91,7 @@ static FILE *openOut(char *name) {
   return outs;
 }
 
-/* gettok:
- * Tokenize a string. Tokens consist of either a non-empty string
+/* Tokenize a string. Tokens consist of either a non-empty string
  * of non-space characters, or all characters between a pair of
  * single or double quotes. As usual, we map
  *   \c -> c
@@ -135,8 +142,7 @@ static char *gettok(char **sp) {
   return s;
 }
 
-/* parseArgs:
- * Split s into whitespace separated tokens, allowing quotes.
+/* Split s into whitespace separated tokens, allowing quotes.
  * Append tokens to argument list and return new number of arguments.
  *
  * @param arg [inout] The current arguments
@@ -207,8 +213,7 @@ static char *dflt_gvprpath(void) {
   return share;
 }
 
-/* resolve:
- * Translate -f arg parameter into a pathname.
+/* Translate -f arg parameter into a pathname.
  * If arg contains '/', return arg.
  * Else search directories in GVPRPATH for arg.
  * Return NULL on error.
@@ -302,8 +307,7 @@ static char *getOptarg(int c, char **argp, int *argip, int argc, char **argv) {
   return rv;
 }
 
-/* doFlags:
- * Process a command-line argument starting with a '-'.
+/* Process a command-line argument starting with a '-'.
  * argi is the index of the next available item in argv[].
  * argc has its usual meaning.
  *
@@ -380,9 +384,7 @@ static void freeOpts(options opts) {
   LIST_FREE(&opts.args);
 }
 
-/* scanArgs:
- * Parse command line options.
- */
+/// parse command line options
 static options scanArgs(int argc, char **argv) {
   char *arg;
   options opts = {0};
@@ -705,9 +707,7 @@ static void travFlat(Gpr_t *state, Expr_t *prog, comp_block *xprog) {
   }
 }
 
-/* doCleanup:
- * Reset node traversal data
- */
+/// reset node traversal data
 static void doCleanup(Agraph_t *g) {
   Agnode_t *n;
   ndata *nd;
@@ -719,9 +719,7 @@ static void doCleanup(Agraph_t *g) {
   }
 }
 
-/* traverse:
- * return true if traversal requires cleanup
- */
+/// return true if traversal requires cleanup
 static bool traverse(Gpr_t *state, Expr_t *prog, comp_block *bp, bool cleanup) {
   if (!state->target) {
     char *target;
@@ -832,8 +830,7 @@ static bool traverse(Gpr_t *state, Expr_t *prog, comp_block *bp, bool cleanup) {
   return cleanup;
 }
 
-/* addOutputGraph:
- * Append output graph to option struct.
+/* Append output graph to option struct.
  * We know uopts and state->outgraph are non-NULL.
  */
 static void addOutputGraph(Gpr_t *state, gvpropts *uopts) {
@@ -877,8 +874,7 @@ typedef struct {
   options opts;
 } gvpr_state_t;
 
-/* gvexitf:
- * Only used if GV_USE_EXIT not set during exeval.
+/* Only used if GV_USE_EXIT not set during exeval.
  * This implies setjmp/longjmp set up.
  */
 static void gvexitf(void *env, int v) {
@@ -905,8 +901,7 @@ static void gverrorf(Expr_t *handle, Exdisc_t *discipline, int level,
   }
 }
 
-/* gvpr_core:
- * Return 0 on success; non-zero on error.
+/* Return 0 on success; non-zero on error.
  *
  * FIX/TODO:
  *  - close non-source/non-output graphs
@@ -927,8 +922,7 @@ static int gvpr_core(int argc, char *argv[], gvpropts *uopts,
     return gs->opts.state;
   }
 
-  if (gs->opts.verbose)
-    gvstart_timer();
+  clock_t start = start_timer();
   gs->prog = parseProg(gs->opts.program, gs->opts.useFile);
   if (gs->prog == NULL) {
     return 1;
@@ -936,11 +930,8 @@ static int gvpr_core(int argc, char *argv[], gvpropts *uopts,
   info.outFile = gs->opts.outFile;
   info.args = gs->opts.args;
   info.errf = gverrorf;
-  if (uopts)
-    info.flags = uopts->flags;
-  else
-    info.flags = 0;
-  if ((uopts->flags & GV_USE_EXIT))
+  info.flags = uopts->flags;
+  if (uopts->flags & GV_USE_EXIT)
     info.exitf = 0;
   else
     info.exitf = gvexitf;
@@ -957,7 +948,7 @@ static int gvpr_core(int argc, char *argv[], gvpropts *uopts,
 
   initGPRState(gs->state);
 
-  if ((uopts->flags & GV_USE_OUTGRAPH)) {
+  if (uopts->flags & GV_USE_OUTGRAPH) {
     uopts->outgraphs = 0;
     uopts->n_outgraphs = 0;
   }
@@ -969,28 +960,27 @@ static int gvpr_core(int argc, char *argv[], gvpropts *uopts,
     }
   }
 
-  bool incoreGraphs = uopts && uopts->ingraphs;
+  bool incoreGraphs = uopts->ingraphs;
 
   if (gs->opts.verbose)
-    fprintf(stderr, "Parse/compile/init: %.2f secs.\n", gvelapsed_sec());
+    fprintf(stderr, "Parse/compile/init: %.2f secs.\n", elapsed_sec(start));
   /* do begin */
   if (gs->xprog->begin_stmt != NULL)
     exeval(gs->xprog->prog, gs->xprog->begin_stmt, gs->state);
 
   /* if program is not null */
   if (gs->xprog->uses_graph) {
-    if (uopts && uopts->ingraphs)
+    if (uopts->ingraphs)
       gs->ing = newIngGraphs(0, uopts->ingraphs, ing_read);
     else
       gs->ing = newIng(0, gs->opts.inFiles, ing_read);
 
-    if (gs->opts.verbose)
-      gvstart_timer();
+    start = start_timer();
     Agraph_t *nextg = NULL;
     for (gs->state->curgraph = nextGraph(gs->ing); gs->state->curgraph;
          gs->state->curgraph = nextg) {
       if (gs->opts.verbose)
-        fprintf(stderr, "Read graph: %.2f secs.\n", gvelapsed_sec());
+        fprintf(stderr, "Read graph: %.2f secs.\n", elapsed_sec(start));
       gs->state->infname = fileName(gs->ing);
       if (gs->opts.readAhead)
         nextg = gs->state->nextgraph = nextGraph(gs->ing);
@@ -1019,7 +1009,7 @@ static int gvpr_core(int argc, char *argv[], gvpropts *uopts,
       if (gs->xprog->endg_stmt != NULL)
         exeval(gs->xprog->prog, gs->xprog->endg_stmt, gs->state);
       if (gs->opts.verbose)
-        fprintf(stderr, "Finish graph: %.2f secs.\n", gvelapsed_sec());
+        fprintf(stderr, "Finish graph: %.2f secs.\n", elapsed_sec(start));
 
       /* if $O == $G and $T is empty, delete $T */
       if (gs->state->outgraph == gs->state->curgraph &&
@@ -1032,7 +1022,7 @@ static int gvpr_core(int argc, char *argv[], gvpropts *uopts,
        */
       if (gs->state->outgraph != NULL &&
           (agnnodes(gs->state->outgraph) || gs->opts.compflags.srcout)) {
-        if (uopts && (uopts->flags & GV_USE_OUTGRAPH))
+        if (uopts->flags & GV_USE_OUTGRAPH)
           addOutputGraph(gs->state, uopts);
         else
           sfioWrite(gs->state->outgraph, gs->opts.outFile);
@@ -1043,12 +1033,11 @@ static int gvpr_core(int argc, char *argv[], gvpropts *uopts,
       gs->state->target = 0;
       gs->state->outgraph = 0;
 
-      if (gs->opts.verbose)
-        gvstart_timer();
+      start = start_timer();
       if (!gs->opts.readAhead)
         nextg = nextGraph(gs->ing);
       if (gs->opts.verbose && nextg != NULL) {
-        fprintf(stderr, "Read graph: %.2f secs.\n", gvelapsed_sec());
+        fprintf(stderr, "Read graph: %.2f secs.\n", elapsed_sec(start));
       }
     }
   }
@@ -1072,6 +1061,11 @@ int gvpr(int argc, char *argv[], gvpropts *uopts) {
   // initialize opts to something that makes freeOpts() a no-op if we fail early
   gvpr_state.opts.outFile = stdout;
 
+  gvpropts DEFAULT_OPTS = {0};
+  if (uopts == NULL) {
+    uopts = &DEFAULT_OPTS;
+  }
+
   int rv = gvpr_core(argc, argv, uopts, &gvpr_state);
 
   // free all allocated resources
