@@ -159,8 +159,7 @@ Dtdisc_t edgeItemDisc = {
     cmpitems,
 };
 
-/* equivEdge:
- * See if we have already encountered an edge between the same
+/* See if we have already encountered an edge between the same
  * node:port pairs. If so, return the earlier edge. If not, 
  * this edge is added to map and returned.
  * We first have to canonicalize the key fields using a lexicographic
@@ -209,8 +208,7 @@ static edge_t *equivEdge(Dt_t * map, edge_t * e)
 }
 
 
-/* makeSelfArcs:
- * Generate loops. We use the library routine makeSelfEdge
+/* Generate loops. We use the library routine makeSelfEdge
  * which also places the labels.
  * We have to handle port labels here.
  * as well as update the bbox from edge labels.
@@ -223,7 +221,7 @@ void makeSelfArcs(edge_t * e, int stepx)
     if (cnt == 1 || Concentrate) {
 	edge_t *edges1[1];
 	edges1[0] = e;
-	makeSelfEdge(edges1, 0, 1, stepx, stepx, &sinfo);
+	makeSelfEdge(edges1, 1, stepx, stepx, &sinfo);
 	if (ED_label(e))
 	    updateBB(agraphof(agtail(e)), ED_label(e));
 	makePortLabels(e);
@@ -233,7 +231,7 @@ void makeSelfArcs(edge_t * e, int stepx)
 	    edges[i] = e;
 	    e = ED_to_virt(e);
 	}
-	makeSelfEdge(edges, 0, cnt, stepx, stepx, &sinfo);
+	makeSelfEdge(edges, cnt, stepx, stepx, &sinfo);
 	for (size_t i = 0; i < cnt; i++) {
 	    e = edges[i];
 	    if (ED_label(e))
@@ -312,8 +310,7 @@ static pointf circumscribed_polygon_corner_about_ellipse(double a, double b,
   return line_intersection(line0, line1);
 }
 
-/* makeObstacle:
- * Given a node, return an obstacle reflecting the
+/* Given a node, return an obstacle reflecting the
  * node's geometry. pmargin specifies how much space to allow
  * around the polygon. 
  * Returns the constructed polygon on success, NULL on failure.
@@ -492,8 +489,7 @@ Ppoly_t *makeObstacle(node_t * n, expand_t* pmargin, bool isOrtho)
     return obs;
 }
 
-/* getPath
- * Construct the shortest path from one endpoint of e to the other.
+/* Construct the shortest path from one endpoint of e to the other.
  * vconfig is a precomputed data structure to help in the computation.
  * If chkPts is true, the function finds the polygons, if any, containing
  * the endpoints and tells the shortest path computation to ignore them. 
@@ -528,8 +524,7 @@ static void makePolyline(edge_t * e) {
     addEdgeLabels(e);
 }
 
-/* makeSpline:
- * Construct a spline connecting the endpoints of e, avoiding the npoly
+/* Construct a spline connecting the endpoints of e, avoiding the npoly
  * obstacles obs.
  * The resultant spline is attached to the edge, the positions of any 
  * edge labels are computed, and the graph's bounding box is recomputed.
@@ -540,7 +535,6 @@ static void makePolyline(edge_t * e) {
  */
 void makeSpline(edge_t *e, Ppoly_t **obs, int npoly, bool chkPts) {
     Ppolyline_t line, spline;
-    Pvector_t slopes[2];
     int i;
     int pp, qp;
     Ppoint_t p, q;
@@ -561,8 +555,7 @@ void makeSpline(edge_t *e, Ppoly_t **obs, int npoly, bool chkPts) {
 
     size_t n_barriers;
     make_barriers(obs, npoly, pp, qp, &barriers, &n_barriers);
-    slopes[0].x = slopes[0].y = 0.0;
-    slopes[1].x = slopes[1].y = 0.0;
+    Pvector_t slopes[2] = {0};
     if (Proutespline(barriers, n_barriers, line, slopes, &spline) < 0) {
 	agerrorf("makeSpline: failed to make spline edge (%s,%s)\n", agnameof(agtail(e)), agnameof(aghead(e)));
 	return;
@@ -713,8 +706,7 @@ static int spline_edges_(graph_t *g, expand_t *pmargin, int edgetype) {
     return 0;
 }
 
-/* splineEdges:
- * Main wrapper code for generating edges.
+/* Main wrapper code for generating edges.
  * Sets desired separation. 
  * Coalesces equivalent edges (edges * with the same endpoints). 
  * It then calls the edge generating function, and marks the
@@ -771,8 +763,7 @@ splineEdges(graph_t * g, int (*edgefn) (graph_t *, expand_t*, int),
     return 0;
 }
 
-/* spline_edges1:
- * Construct edges using default algorithm and given splines value.
+/* Construct edges using default algorithm and given splines value.
  * Return 0 on success.
  */
 int spline_edges1(graph_t * g, int edgetype)
@@ -780,8 +771,7 @@ int spline_edges1(graph_t * g, int edgetype)
   return splineEdges(g, spline_edges_, edgetype);
 }
 
-/* spline_edges0:
- * Sets the graph's aspect ratio.
+/* Sets the graph's aspect ratio.
  * Check splines attribute and construct edges using default algorithm.
  * If the splines attribute is defined but equal to "", skip edge routing.
  * 
@@ -824,8 +814,7 @@ shiftClusters (graph_t * g, pointf offset)
     GD_bb(g).LL.y -= offset.y;
 }
 
-/* spline_edges:
- * Compute bounding box, translate graph to origin,
+/* Compute bounding box, translate graph to origin,
  * then construct all edges.
  */
 void spline_edges(graph_t * g)
@@ -845,8 +834,7 @@ void spline_edges(graph_t * g)
     spline_edges0(g, true);
 }
 
-/* scaleEdge:
- * Scale edge by given factor.
+/* Scale edge by given factor.
  * Assume ED_spl != NULL.
  */
 static void scaleEdge(edge_t * e, double xf, double yf)
@@ -903,9 +891,7 @@ static void scaleEdge(edge_t * e, double xf, double yf)
     }
 }
 
-/* scaleBB:
- * Scale bounding box of clusters of g by given factors.
- */
+/// scale bounding box of clusters of g by given factors
 static void scaleBB(graph_t * g, double xf, double yf)
 {
     int i;
@@ -924,8 +910,7 @@ static void scaleBB(graph_t * g, double xf, double yf)
 	scaleBB(GD_clust(g)[i], xf, yf);
 }
 
-/* translateE:
- * Translate edge by offset.
+/* Translate edge by offset.
  * Assume ED_spl(e) != NULL
  */
 static void translateE(edge_t * e, pointf offset)
@@ -1015,8 +1000,7 @@ void neato_translate(Agraph_t * g)
     translateG(g, ll);
 }
 
-/* _neato_set_aspect;
- * Assume all bounding boxes are correct.
+/* Assume all bounding boxes are correct.
  * Return false if no transform is performed. This includes
  * the possibility that a translation was done.
  */
@@ -1102,8 +1086,7 @@ static bool _neato_set_aspect(graph_t * g)
 	return false;
 }
 
-/* neato_set_aspect:
- * Sets aspect ratio if necessary; real work done in _neato_set_aspect;
+/* Sets aspect ratio if necessary; real work done in _neato_set_aspect;
  * This also copies the internal layout coordinates (ND_pos) to the 
  * external ones (ND_coord).
  *
