@@ -21,6 +21,8 @@
 
 #pragma once
 
+#include <util/api.h>
+
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -31,12 +33,6 @@ extern "C" {
 #include <neatogen/digcola.h>
 #include <stdbool.h>
 
-#if !defined(__CYGWIN__) && defined(__GNUC__) && !defined(__MINGW32__)
-#define INTERNAL __attribute__((visibility("hidden")))
-#else
-#define INTERNAL /* nothing */
-#endif
-
 typedef struct CMajEnvVPSC {
 	float **A;
 	int nv;   /* number of actual vars */
@@ -54,12 +50,12 @@ typedef struct CMajEnvVPSC {
 	float *fArray3;
 } CMajEnvVPSC;
 
-INTERNAL CMajEnvVPSC* initCMajVPSC(int n, float *packedMat, vtx_data* graph, ipsep_options *opt, int diredges);
+PRIVATE CMajEnvVPSC* initCMajVPSC(int n, float *packedMat, vtx_data* graph, ipsep_options *opt, int diredges);
 
-INTERNAL int constrained_majorization_vpsc(CMajEnvVPSC*, float*, float*, int);
+PRIVATE int constrained_majorization_vpsc(CMajEnvVPSC*, float*, float*, int);
 
-INTERNAL void deleteCMajEnvVPSC(CMajEnvVPSC *e);
-INTERNAL void generateNonoverlapConstraints(
+PRIVATE void deleteCMajEnvVPSC(CMajEnvVPSC *e);
+PRIVATE void generateNonoverlapConstraints(
         CMajEnvVPSC* e,
         float nsizeScale,
         float** coords,
@@ -68,7 +64,7 @@ INTERNAL void generateNonoverlapConstraints(
 	ipsep_options* opt
 );
 
-INTERNAL void removeoverlaps(int,float**,ipsep_options*);
+PRIVATE void removeoverlaps(int,float**,ipsep_options*);
 
 typedef struct {
 	int *nodes;
@@ -78,10 +74,8 @@ typedef struct {
 /*
  * unpack the "ordering" array into an array of DigColaLevel (as defined above)
  */
-INTERNAL DigColaLevel* assign_digcola_levels(int *ordering, int n, int *level_inds, int num_divisions);
-INTERNAL int get_num_digcola_constraints(DigColaLevel *levels, int num_levels);
-
-#undef INTERNAL
+PRIVATE DigColaLevel* assign_digcola_levels(int *ordering, int n, int *level_inds, int num_divisions);
+PRIVATE int get_num_digcola_constraints(DigColaLevel *levels, int num_levels);
 
 #endif 
 
