@@ -3,7 +3,7 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -25,7 +25,6 @@ extern "C" {
     typedef COORD **array2;
 
 #define EQ(p,q)		((p.x == q.x) && (p.y == q.y))
-#define NIL(p)		((p)0)
 
     struct vconfig_s {
 	int Npoly;
@@ -38,21 +37,26 @@ extern "C" {
 	/* this is computed from the above */
 	array2 vis;
     };
-#ifdef _WIN32
-#ifndef PATHPLAN_EXPORTS
-#define extern __declspec(dllimport)
+#ifdef GVDLL
+#ifdef PATHPLAN_EXPORTS
+#define VIS_API __declspec(dllexport)
+#else
+#define VIS_API __declspec(dllimport)
 #endif
 #endif
-/*end visual studio*/
 
-	extern COORD *ptVis(vconfig_t *, int, Ppoint_t);
-    extern bool directVis(Ppoint_t, int, Ppoint_t, int, vconfig_t *);
-    extern void visibility(vconfig_t *);
-    extern int *makePath(Ppoint_t p, int pp, COORD * pvis,
+#ifndef VIS_API
+#define VIS_API /* nothing */
+#endif
+
+	VIS_API COORD *ptVis(vconfig_t *, int, Ppoint_t);
+    VIS_API bool directVis(Ppoint_t, int, Ppoint_t, int, vconfig_t *);
+    VIS_API void visibility(vconfig_t *);
+    VIS_API int *makePath(Ppoint_t p, int pp, COORD * pvis,
 			 Ppoint_t q, int qp, COORD * qvis,
 			 vconfig_t * conf);
 
-#undef extern
+#undef VIS_API
 
 #ifdef __cplusplus
 }
