@@ -3,7 +3,7 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -21,17 +21,9 @@ extern "C" {
 
 #include "config.h"
 
-#ifdef HAVE_SYS_TYPES_H
-#   include <sys/types.h>
-#endif // HAVE_SYS_TYPES_H
-
 #include <limits.h>
-#include <stdarg.h>
 #include <stdio.h>
-
-#define Sflong_t	long long
-#define Sfulong_t	unsigned long long
-#define Sfdouble_t	long double
+#include <sys/types.h>
 
 /* formatting environment */
     typedef struct _sffmt_s Sffmt_t;
@@ -40,7 +32,6 @@ extern "C" {
 	Sffmtext_f extf;	/* function to process arguments        */
 
 	char *form;		/* format string to stack               */
-	va_list args;		/* corresponding arg list               */
 
 	int fmt;		/* format character                     */
 	ssize_t size;		/* object size                          */
@@ -77,8 +68,7 @@ extern "C" {
      extern ssize_t _Sfi;
 
     extern int sfprint(FILE*, Sffmt_t *format);
-    extern int sfscanf(FILE*, ...);
-    extern int sfvscanf(FILE*, va_list);
+    extern int sfvscanf(FILE *, Sffmt_t *format);
 
 /* miscellaneous function analogues of fast in-line functions */
     extern ssize_t sfslen(void);
