@@ -669,7 +669,7 @@ void core_loadimage_xdot(GVJ_t * job, usershape_t *us, boxf b, bool filled)
     xdot_str (job, "", us->name);
 }
 
-gvrender_engine_t dot_engine = {
+static gvrender_engine_t dot_engine = {
     0,				/* dot_begin_job */
     0,				/* dot_end_job */
     dot_begin_graph,
@@ -702,7 +702,7 @@ gvrender_engine_t dot_engine = {
     0,				/* dot_library_shape */
 };
 
-gvrender_engine_t xdot_engine = {
+static gvrender_engine_t xdot_engine = {
     0,				/* xdot_begin_job */
     0,				/* xdot_end_job */
     dot_begin_graph,
@@ -735,7 +735,7 @@ gvrender_engine_t xdot_engine = {
     0,				/* xdot_library_shape */
 };
 
-gvrender_features_t render_features_dot = {
+static gvrender_features_t render_features_dot = {
     GVRENDER_DOES_TRANSFORM,	/* not really - uses raw graph coords */  /* flags */
     0.,                         /* default pad - graph units */
     NULL,			/* knowncolors */
@@ -743,7 +743,7 @@ gvrender_features_t render_features_dot = {
     COLOR_STRING,		/* color_type */
 };
 
-gvrender_features_t render_features_xdot = {
+static gvrender_features_t render_features_xdot = {
     GVRENDER_DOES_TRANSFORM 	/* not really - uses raw graph coords */  
 	| GVRENDER_DOES_MAPS
 	| GVRENDER_DOES_TARGETS
@@ -754,14 +754,14 @@ gvrender_features_t render_features_xdot = {
     RGBA_BYTE,		/* color_type */
 };
 
-gvdevice_features_t device_features_canon = {
+static gvdevice_features_t device_features_canon = {
     LAYOUT_NOT_REQUIRED,	/* flags */
     {0.,0.},			/* default margin - points */
     {0.,0.},                    /* default height, width - device units */
     {72.,72.},			/* default dpi */
 };
 
-gvdevice_features_t device_features_dot = {
+static gvdevice_features_t device_features_dot = {
     0,				/* flags */
     {0.,0.},			/* default margin - points */
     {0.,0.},			/* default page width, height - points */
