@@ -3,20 +3,22 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
-#include <cgraph/unreachable.h>
+#include <stdbool.h>
+#include <stddef.h>
+#include <stdint.h>
+#include <util/alloc.h>
+#include <util/unreachable.h>
 #include "tcldot.h"
 
 // Agiddisc functions
 static void *myiddisc_open(Agraph_t *g, Agdisc_t *disc) {
-    ictx_t *ictx = (ictx_t *)disc;
-    gctx_t *gctx;
-
-    gctx = malloc(sizeof(gctx_t));
+    ictx_t *const ictx = (ictx_t *)((char *)disc - offsetof(ictx_t, mydisc));
+    gctx_t *const gctx = gv_alloc(sizeof(gctx_t));
     gctx->g = g;
     gctx->ictx = ictx;
     return gctx;
@@ -33,7 +35,7 @@ static long myiddisc_map(void *state, int objtype, char *str, uint64_t *id, int
             s = agstrdup(gctx->g, str);
         else
             s = agstrbind(gctx->g, str);
-        *id = (uint64_t) s;
+        *id = (uint64_t)(uintptr_t)s;
     } else {
         *id = ictx->ctr;  /* counter maintained in per-interp space, so that
 		ids are unique across all graphs in the interp */
@@ -41,13 +43,7 @@ static long myiddisc_map(void *state, int objtype, char *str, uint64_t *id, int
     }
     return 1;
 }
-/* we don't allow users to explicitly set IDs, either */
-static long myiddisc_alloc(void *state, int objtype, uint64_t request_id) {
-    (void)state;
-    (void)objtype;
-    (void)request_id;
-    return 0;
-}
+
 static void myiddisc_free(void *state, int objtype, uint64_t id) {
     (void)objtype;
 
@@ -66,19 +62,17 @@ static void myiddisc_free(void *state, int objtype, uint64_t id) {
     Tcl_DeleteCommand(ictx->interp, buf);
 */
     if (id % 2 == 0)
-        agstrfree(gctx->g, (char *) id);
+        agstrfree(gctx->g, (char *)(uintptr_t)id, false);
 }
 static char *myiddisc_print(void *state, int objtype, uint64_t id) {
     (void)state;
     (void)objtype;
     if (id % 2 == 0)
-        return (char *) id;
+        return (char *)(uintptr_t)id;
     else
         return "";
 }
-static void myiddisc_close(void *state) {
-    free(state);
-}
+
 static void myiddisc_idregister(void *state, int objtype, void *obj) {
     gctx_t *gctx = state;
     ictx_t *ictx = gctx->ictx;
@@ -92,18 +86,13 @@ static void myiddisc_idregister(void *state, int objtype, void *obj) {
         case AGOUTEDGE: proc=edgecmd; break;
         default: UNREACHABLE();
     }
-#ifndef TCLOBJ
-    Tcl_CreateCommand(interp, obj2cmd(obj), proc, (ClientData) gctx, (Tcl_CmdDeleteProc *) NULL);
-#else
-    Tcl_CreateObjCommand(interp, obj2cmd(obj), proc, (ClientData) gctx, (Tcl_CmdDeleteProc *) NULL);
-#endif           
+    Tcl_CreateCommand(interp, obj2cmd(obj), proc, (ClientData) gctx, NULL);
 }
 Agiddisc_t myiddisc = {
     myiddisc_open,
     myiddisc_map,
-    myiddisc_alloc,
     myiddisc_free,
     myiddisc_print,
-    myiddisc_close,
+    free,
     myiddisc_idregister
 };
